/*
Author:
Sreekumar Thaithara Balan 

Name:
control_system.h

Version:
0.7

Last modified:
30-03-2008

Purpose:

1. Defines interface for Acceptance Search
   Check for acceptance in every sample_size updates
   Tunes the sigma to get the traget acceptance
2. Defines interface for Sigma Search
   The sigmas are updated every sample_size iterations by 
   Sigma estimated from parameter values from sample_size iterations
   

Implements:

Todo:

References:

1. Rosethal
2. H. Harrio, E. Saksman and J. Tamminen,

License:
GNU General Public License
This file is part of Exofit.

Exofit is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Exofit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Exofit; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
Boston, MA 02110-1301, USA.

*/

#ifndef CONTROL_SYSTEM_H
#define CONTROL_SYSTEM_H

#include<string>
#include<fstream>
#include<vector>
#include"update.h"


// ACCEPTANCE SEARCH

class Acceptance_search{

 public:
 
  Acceptance_search(State* stat,
		    double scale,
		    long max_iterations,
		    long sample_size,
		    int thin,
		    double target_acceptance,
		    int max_levels,
		    double max_anneal);

  // functions for examining the Control system parameters
  double get_scale() const;
  long get_max_iterations() const;
  long get_sample_size() const;
  int get_thin() const;
  double get_target_acceptance() const;
  int get_max_levels() const;
  double get_max_anneal() const;

  // functions for changing the Control system parameters
  void set_state(State*);
  void set_scale(double);
  void set_max_iterations(long);
  void set_sample_size(long);
  void set_thin(int);
  void set_target_acceptance(double);
  void set_max_levels(int);
  void set_max_anneal(double);

  // search
  int search(std::ofstream& burn_file,bool index);

 private:
  State* st;
  double sc;
  long max_iter;
  long samp_siz;
  int thn;
  double tg_acc;
  int max_lvls;
  double max_annl;
};

class Sigma_search{

 public: 
  Sigma_search(State* stat,
	       double scale,
	       long max_iterations,
	       long sample_size,
	       int thin,
	       int windows);

  // functions for examining the Control system parameters
  double get_scale() const;
  long get_max_iterations() const;
  long get_sample_size() const;
  int get_thin() const;
  int get_max_windows() const;

  // functions for changing the Control system parameters
  void set_state(State*);
  void set_scale(double);
  void set_max_iterations(long);
  void set_sample_size(long);
  void set_thin(int);
  void set_max_windows(int);

  // search
  int search(std::ofstream& burn_file,
	      bool index_b,
	      std::ofstream& diag_file,
	      bool index_d);


 private:
  State* st;
  double sc;
  long max_iter;
  long samp_siz;
  int max_wind;
  int thn;
};

class Mean_monitored_run{

 public: 
  Mean_monitored_run(State* stat,
		     long max_iterations,
		     long sample_size,
		     int thin,
		     int windows);

  // functions for examining the Control system parameters
  double get_scale() const;
  long get_max_iterations() const;
  long get_sample_size() const;
  int get_thin() const;
  int get_max_windows() const;

  // functions for changing the Control system parameters
  void set_state(State*);
  void set_scale(double);
  void set_max_iterations(long);
  void set_sample_size(long);
  void set_thin(int);
  void set_max_windows(int);

  // run
  int run(std::ofstream& out_fil, bool index);

 private:
  State* st;
  long max_iter;
  long samp_siz;
  int max_wind;
  int thn;
};

// INLINES FOR CLASS ACCEPTANCE SEARCH

inline Acceptance_search::Acceptance_search(State* stat,
					    double scale,
					    long max_iterations,
					    long sample_size,
					    int thin,
					    double target_acceptance,
					    int max_levels,
					    double max_anneal)
{
  st=stat;
  sc=scale;
  max_iter=max_iterations;
  samp_siz=sample_size;
  thn=thin;
  tg_acc=target_acceptance;
  max_lvls=max_levels;
  max_annl=max_anneal;
}
inline double Acceptance_search::get_scale() const
{
  return sc;
}
inline long Acceptance_search::get_max_iterations() const
{
  return max_iter;
}
inline long Acceptance_search::get_sample_size() const
{
  return samp_siz;
}
inline int Acceptance_search::get_thin() const
{
  return thn;
}
inline double Acceptance_search::get_target_acceptance() const
{
  return tg_acc;
}
inline int Acceptance_search::get_max_levels() const
{
  return max_lvls;
}
inline double Acceptance_search::get_max_anneal() const
{
  return max_annl;
}
inline void Acceptance_search::set_scale(double n)
{
  sc=n;
}
inline void Acceptance_search::set_max_iterations(long i)
{
  max_iter=i;
}
inline void Acceptance_search::set_sample_size(long i)
{
  samp_siz=i;
}
inline void Acceptance_search::set_thin(int i)
{
  thn=i;
}
inline void Acceptance_search::set_target_acceptance(double n)
{
  tg_acc=n;
}
inline void Acceptance_search::set_max_levels(int i)
{
  max_lvls=i;
}
inline void Acceptance_search::set_max_anneal(double n)
{
  max_annl=n;
}

// INLINES FOR CLASS SIGMA SEARCH

inline Sigma_search::Sigma_search(State* stat,
				  double scale,
				  long max_iterations,
				  long sample_size,
				  int windows,
				  int thin)
{
  st=stat;
  sc=scale;
  max_iter=max_iterations;
  samp_siz=sample_size;
  max_wind=windows;
  thn=thin;
}
inline double Sigma_search::get_scale() const
{
  return sc;
}
inline long Sigma_search::get_max_iterations() const
{
  return max_iter;
}
inline long Sigma_search::get_sample_size() const
{
  return samp_siz;
}
inline int Sigma_search::get_thin() const
{
  return thn;
}
inline int Sigma_search::get_max_windows() const
{
  return max_wind;
}
inline void Sigma_search::set_scale(double n)
{
  sc=n;
}
inline void Sigma_search::set_max_iterations(long i)
{
  max_iter=i;
}
inline void Sigma_search::set_sample_size(long i)
{
  samp_siz=i;
}
inline void Sigma_search::set_thin(int i)
{
  thn=i;
}
inline void Sigma_search::set_max_windows(int i)
{
  max_wind=i;
}

// INLINES FOR CLASS MEAN MONITORED RUN

inline Mean_monitored_run::Mean_monitored_run(State* stat,
					      long max_iterations,
					      long sample_size,
					      int windows,
					      int thin)
{
  st=stat;
  max_iter=max_iterations;
  samp_siz=sample_size;
  max_wind=windows;
  thn=thin;
}

inline long Mean_monitored_run::get_max_iterations() const
{
  return max_iter;
}
inline long Mean_monitored_run::get_sample_size() const
{
  return samp_siz;
}
inline int Mean_monitored_run::get_thin() const
{
  return thn;
}
inline int Mean_monitored_run::get_max_windows() const
{
  return max_wind;
}

inline void Mean_monitored_run::set_max_iterations(long i)
{
  max_iter=i;
}
inline void Mean_monitored_run::set_sample_size(long i)
{
  samp_siz=i;
}
inline void Mean_monitored_run::set_thin(int i)
{
  thn=i;
}
inline void Mean_monitored_run::set_max_windows(int i)
{
  max_wind=i;
}

#endif //CONTROL_SYSTEM_H
