/*
Author:
Sreekumar Thaithara Balan

Name:
mcmc.cc

Version:
0.7

Last modified:
30-03-2008

Purpose:
1. Definition of all the parameters,states,bonds,and control system
2. Define data files

Implements:
1. Mcmc::run()->mcmc.h
2. Parallel_tempering::initialise()->mcmc.h
3. Parallel_tempering::update()->mcmc.h

Todo:
1. Sequential update in parallel tempering

References:
1. Gregory Liu...
2. T. L. Graves

License:
GNU General Public License
This file is part of Exofit.

Exofit is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Exofit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Exofit; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
Boston, MA 02110-1301, USA.

*/


#include<iostream>
#include<cmath>
#include"mcmc.h"
#include"data.h"
#include"density.h"
#include"ctrl_sys.h"



void Mcmc::run()
{

  // define the parameters here
  Parameter period("T",0.2,7500.0,15000.0,1500.0);
  Parameter amplitude("K",0.000001,1000.0,2000.0,200.0);
  Parameter sys_velocity("V",-2000.0,0.0,2000.0,400.0);
  Parameter eccentricity("e",0.0,0.5,0.99999,0.1);
  Parameter long_periastron("w",0.0,3.14159,2*3.14159,0.628);
  Parameter periastron_pass("X",0.0,0.5,0.9999,0.1);
  Parameter noise_factor("s",0.0,1000.0,2000.0,200.0);

  //define the prior densities
  Uniform uniform;
  Jeffreys jeffreys;
  Modified_jeffreys mod_jeff(1.0);

  // Define the radial velocity data and read that file
  Data rv_data(in_fil,3);
  rv_data.read_file();
  const int max_dat=rv_data.get_n_entries();  

  for(int i=0;i<max_dat;++i)
    {
      std::cout<<rv_data.get_data_entry(i,0)<<"\t";
      std::cout<<rv_data.get_data_entry(i,1)<<"\t";
      std::cout<<rv_data.get_data_entry(i,2)<<"\n";
    }


  //rv_data.print_entries();
  std::cout<<"number of entries="<<rv_data.get_n_entries()<<"\n";
  
  // now make the data set start from zero  
  double start_val=rv_data.get_data_entry(0,0);
  for(int i=0;i<max_dat;++i)
    {
      double offset=rv_data.get_data_entry(i,0)-start_val;
      rv_data.set_data_entry(i,0,offset);
    }

  
  //define the likelihood
  Gaussian gauss(&rv_data);

  //define an mcmc state here
  State mcstate;

  //add bonds
  mcstate.add_bond(period,&jeffreys);
  mcstate.add_bond(amplitude,&mod_jeff);
  mcstate.add_bond(sys_velocity,&uniform);
  mcstate.add_bond(eccentricity,&uniform);
  mcstate.add_bond(long_periastron,&uniform);
  mcstate.add_bond(periastron_pass,&uniform);
  mcstate.add_bond(noise_factor,&mod_jeff);
  //add likelihood
  mcstate.add_likelihood(&gauss);

  int n_para=7;// number of parameters in the state should be specified 

  // now if there exists a state.dat
  char st_data_file[]="state.dat";
  char* d_st;
  d_st=st_data_file;

  Data state_data(n_para,true);// state file always has row label
  if(take_state_data==true)
    {
      state_data.set_data_file(d_st);
      state_data.read_file();

      std::cout<<"**********************************************************\n";
      std::cout<<"                INPUTS FROM \"state.dat\"                 \n";
      std::cout<<"**********************************************************\n";
      state_data.print_head();
      state_data.print_entries(); 
      std::cout<<"**********************************************************\n";
      
      //now read the data into the state vector
      std::string hd_tmp;// dummy for parameter labels
      double var_tmp;// dummy for parameter values
      for(int i=0;i<n_para;++i)
	{
	  hd_tmp=state_data.get_head(i);
	  mcstate.set_para_label(i,hd_tmp);
	  
	  var_tmp=state_data.get_data_entry(0,i);// min is the 1st entry in data
	  mcstate.set_para_min(i,var_tmp);

	  var_tmp=state_data.get_data_entry(1,i);// get the val 2nd entry
	  mcstate.set_para_val(i,var_tmp);
	  mcstate.set_para_prop_val(i,var_tmp);

	  var_tmp=state_data.get_data_entry(2,i);//get the max 3rd entry
	  mcstate.set_para_max(i,var_tmp);

	  var_tmp=state_data.get_data_entry(3,i);//get the step size 4th entry 
	  mcstate.set_para_step_size(i,var_tmp);
	  
	}
    } 

  // random numbers for state
  const gsl_rng_type* Ty;
  gsl_rng* rm;
  gsl_rng_env_setup();
  Ty=gsl_rng_default;
  rm=gsl_rng_alloc(Ty);
  unsigned long int sd=gsl_rng_uniform_int(rm,2500);
  mcstate.set_rands(gsl_rng_default,sd);
 

  //open the burn file
  std::ofstream brn_file;
  brn_file.open("burn.dat",std::ios::trunc);
  brn_file.close();
  brn_file.open("burn.dat",std::ios::app);

  //open the diagnostics file
  std::ofstream dg_file;
  dg_file.open("diag.dat",std::ios::trunc);
  dg_file.close();
  dg_file.open("diag.dat",std::ios::app);

  /*
  Acceptance_search acc_ser(&mcstate,
			    0.1,
			    500000,
			    1000,
			    60,
			    0.1,
			    5,
			    0.4);
  acc_ser.search(brn_file,false);
  */
  std::cout<<"**********************************************************\n";

  std::cout<<"Searching for ideal step sizes\n";
 
  Sigma_search sig_ser(&mcstate,
		       0.12,//sclae
		       1000000,//max iterations
		       10000,// sample size
		       100,// thin
		       30);//windows
  int ser_succ=sig_ser.search(brn_file,false,dg_file,true);

  if(ser_succ==1)
    std::cout<<"CUATION ! Unsuccessful\n";
  else
    std::cout<<"Successful\n";

  std::cout<<"**********************************************************\n";

  brn_file.close();
  dg_file.close();


  //  define the output file
  std::ofstream out_file;
  out_file.open("extract.dat",std::ios::trunc); // clear data from outfile
  out_file.close();
  out_file.open("extract.dat",std::ios::app);// open in append mode
  mcstate.write_labels(out_file,false);	

  std::cout<<"Starting the main iteraions: Mean is monitored\n";

  Mean_monitored_run m_run(&mcstate,
			   1000000,// max iterations
			   10000,//sample size
			   100,// thin
			   50);// windows
  int m_succ=m_run.run(out_file,false);

  if(m_succ==1)
    {
      std::cout<<"CUATION ! One or more parameters shows divergence\n";
      std::cout<<"You may start the program with new random seed\n";
    }
  else
    std::cout<<"All the parameters seem to be converged\n";

  std::cout<<"**********************************************************\n";

  out_file.close();

}// end of mcmc

// PARALLEL TEMPERING

void  Parallel_tempering::initialise(State st)
{

  // set up the radom numbers and GSL ENV SETUP 
  gsl_rng_env_setup();
  typ=gsl_rng_default;
  rng=gsl_rng_alloc(typ);
  long unsigned int seed;

  // find out the temperatures on a linear scale
  double d_tmp_lvl=(1-max_temp)/(temp_levels-1);

  int j;
  for(j=0;j<temp_levels;++j)
    {
      temper_vector.push_back(st);// add states to the temp vector
      seed=gsl_rng_uniform_int(rng,2500);
      temper_vector[j].set_rands(gsl_rng_default,seed);
      temper_vector[j].set_temperature(max_temp+j*d_tmp_lvl);
      //std::cout<<"temp level"<<j<<"="<<temper_vector[j].get_temperature()<<"\n"; 
   }

  for(j=0;j<3;++j)
    {
      rn[j]=gsl_rng_alloc(typ);
      seed=gsl_rng_uniform_int(rng,200);
      gsl_rng_set(rn[j],seed);
    }
}

void Parallel_tempering::update(long int n_iter,
				long int n_burn,
				int thn,
				Update* upd,
				std::ofstream& ofstrm,
				bool index)
{
  acceptance=0;
  for(int k=0;k<n_iter;++k)
    {
      int j;
      for(j=0;j<temp_levels;++j)
	{
	  // calculate acceptace for the temp=1 level;
	  if(j==temp_levels-1)
	    {
	      acceptance=acceptance+temper_vector[j].update(upd);
	    }

	  //otherwise just update dont store the acceptance
	  int acc=temper_vector[j].update(upd);

	}// temp levels


      //propose a swap
      double prop=gsl_rng_uniform(rn[0]);
      if(prop<=1/n_swap)
	{

	  //pick a chain
	  //gsl_rng_uniform_int(r,n) returns a value between 0 and n-1
	  long unsigned int pick=gsl_rng_uniform_int(rn[1],temp_levels-1)+1;

	  //now calculate the ratio
	  double level_1=temper_vector[pick-1].get_temperature();
	  double lik_1=temper_vector[pick-1].current_likelihood();
	  double level=temper_vector[pick].get_temperature();
	  double lik=temper_vector[pick].current_likelihood();

	  double log_swap_ratio;
	  log_swap_ratio=level_1*lik + level*lik_1 - level_1*lik_1 - level*lik;
	  double swap_ratio=exp(log_swap_ratio);
	  double swap=gsl_rng_uniform(rn[2]);
	  
	  // swap process
	  if(swap<=swap_ratio)
	    {
	      double swap_val_1,swap_val;
	      int para_max=temper_vector[0].get_no_state();
	      for(int ii=0;ii<para_max;++ii)
		{
		  swap_val=temper_vector[pick].get_para_val(ii);
		  swap_val_1=temper_vector[pick-1].get_para_val(ii);
		  temper_vector[pick].set_para_val(ii,swap_val_1);
		  temper_vector[pick-1].set_para_val(ii,swap_val);
		}
	    }
	  temper_vector[pick].set_calc_strength(true);
	  temper_vector[pick-1].set_calc_strength(true);
	
	}

      // thin and burn 
      if(mcmc_count>n_burn && (k % thn)==0)
	{
	  if(index==true)
	    temper_vector[temp_levels-1].write_extract(ofstrm,mcmc_count);
	  else
	    temper_vector[temp_levels-1].write_extract(ofstrm);
	}

      ++mcmc_count;
    }// major iteration
  
  //std::cout<<"acceptace="<<acceptance<<"\n";

}

void Parallel_tempering::set_state_step_size(int i,double n)// for ith para
{
  for(int j=0;j<temper_vector.size();++j)
    {
      temper_vector[j].set_para_step_size(i, n);
      //std::cout<<"temp_vector["<<j<<"]="<<temper_vector[i].get_para_step_size(i)<<"\n";
    }
}
