/*
Author:
Sreekumar Thaithara Balan 

Name:
update.h

Version:
0.7

Last modified:
30-03-2008

Purpose:
1. Defines the following inerfaces
   a. Update
   b. Parameter
   c. State

Implements:

Todo:
1. Reduce the dependencies between these classes

References:
1. T. L. Graves

License:
GNU General Public License
This file is part of Exofit.

Exofit is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Exofit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Exofit; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
Boston, MA 02110-1301, USA.

*/

#ifndef UPDATE_H
#define UPDATE_H

#include<vector>
#include<fstream>
#include<string>
#include<gsl/gsl_rng.h>
#include<gsl/gsl_randist.h>
#include"density.h"


class Update{

 public:
  Update();
  Update(int,bool);
  Update(int max,int i,bool bl);
  void assign(int max,int i,bool bl);
  void assign(int,bool);
  void assign_all(bool);
  bool check(int);
  int size();


 private:
  std::vector<bool> update_vector;
};

class Parameter{

 public:

  class Bad_parameter{};/************** IN THE NEXT VERSION ***********/
  
  //constructors
  Parameter();
  Parameter(char*,double,double,double,double);

  //functions for examining the state of variables in Parameter class
  std::string get_label() const;
  double get_min() const;
  double get_val() const;
  double get_prop_val() const;
  double get_max() const;
  double get_step_size() const;


  //functions for changing the state of variables in Parameter class
  void set_label(std::string n);
  void set_min(double n);
  void set_val(double n);
  void set_prop_val(double n);
  void set_max(double n);
  void set_step_size(double n);
  
  //function for setting up radom numbers
  void set_rand_type(const gsl_rng_type* );
  void set_rand_seed(long unsigned int );

  //set a new value for the prop
  void propose();
  //get the range of parameters
  double get_range() const;

 private:

  std::string hd;        //heading or a proxy name for the parameter
  double min;       //minimum value of the parameter
  double v;         //current state of the parameter
  double prop;      //proposed value of the parameter-> used  in MCMC
  double max;       //maximum value of the parameter
  double ss;        //step size of the parameter-> used in Proposal and MCMC

  const gsl_rng_type* T;
  gsl_rng* r;

};


class State{

 public:

  //constructor
  State();

  //functions for examining the ith variable in a state
  std::string get_para_label(int i) const;
  double get_para_min(int i) const;
  double get_para_val(int i) const;
  double get_para_prop_val(int i) const;
  double get_para_max(int i) const;
  double get_para_step_size(int i) const;
  bool get_calc_strength() const;
  double get_strength() const;
  double get_temperature() const;

  //functions for modifying the ith varable in a state
  void set_para_label(int i,std::string head);
  void set_para_min(int i,double n);
  void set_para_val(int i,double n);
  void set_para_prop_val(int i,double n);
  void set_para_max(int i,double n);
  void set_para_step_size(int i,double n);

  void set_rands(const gsl_rng_type* typ,long unsigned int SD);
  void set_calc_strength(bool b);
  void set_strength(double n);
  void set_temperature(double n);

  // definition of the state consists of 
  // 1. finite number of parameters AND( 2. likelihood  AND/OR 3 prior)
  void add_bond(Parameter);
  void add_bond(Parameter,Density*);
  void add_likelihood(Likelihood*);
  int get_no_state();  

  //functions need for the Metropolis-Hastings
  double get_para_range(int i) const; 
  void para_propose(int i);
  int update(Update* updt);  
  double current_strength();
  double current_likelihood();
  double new_strength();
  bool out_of_range();

  //functions for writing the extract file
  void write_labels(std::ofstream&,bool ind);
  void write_extract(std::ofstream&);
  void write_extract(std::ofstream&,long n);

  static Null_density null; // if no prior is included in add_bond -> add this

 private:
  std::vector<Parameter> state_vector;
  std::vector<Density*> prior_vector;
  Likelihood* lik;
  const gsl_rng_type *T;
  gsl_rng* R;

  bool calc_strength;
  double strength;
  double temperature;
};

/*************** INLINES FOR UPDATE CLASS ********************************/
inline Update::Update()
{
}
inline Update::Update(int max,bool bl)
{
  update_vector.assign(max,bl);
}

inline void Update::assign(int i,bool bl)
{
  update_vector[i]=bl;
}
inline bool Update::check(int i)
{
  return update_vector[i];
}
inline int Update::size()
{
  return update_vector.size();
}

/****************** INLINES FOR PARAMETER CLASS ***************************/
inline Parameter::Parameter()
{
  hd="Para";
  min=0.01;
  v=0.05;
  prop=0.05;
  max=0.999;
  ss=0.004;
}//defalut values

inline Parameter::Parameter(char head[],
			    double val_min,
			    double val,
			    double val_max,
			    double st_siz)
{
  hd=head;
  min=val_min;
  v=val;
  prop=val;
  max=val_max;
  ss=st_siz;// step size should be given as n/100 format
}

inline std::string Parameter::get_label() const
{
  return hd;
}
inline double Parameter::get_min() const
{
  return min;
}
inline double Parameter::get_val() const
{
  return v;
}
inline double Parameter::get_prop_val() const
{
  return prop;
}
inline double Parameter::get_max() const
{
  return max;
}
inline double Parameter::get_step_size() const
{
  return ss;
}
inline void Parameter::set_label(std::string head)
{
  hd=head;
}
inline void Parameter::set_min(double n)
{
  min=n;
}
inline void Parameter::set_val(double n)
{
  v=n;
}
inline void Parameter::set_prop_val(double n)
{
  prop=n;
}
inline void Parameter::set_max(double n)
{
  max=n;
}
inline void Parameter::set_step_size(double n)
{
  ss=n;
}
inline double Parameter::get_range() const
{
  return max-min;
}

/****************** INLINES FOR STATE CLASS ***************************/
inline State::State()
{
  calc_strength=true; // just a tricj to put the value in the 1st iteration
  temperature=1.0;
}
inline std::string State::get_para_label(int i) const
{
  return state_vector[i].get_label();
}
inline double State::get_para_min(int i)const
{
  return state_vector[i].get_min();
}
inline double State::get_para_val(int i)const
{
  return state_vector[i].get_val();
}
inline double State::get_para_prop_val(int i)const
{
  return state_vector[i].get_prop_val();
}
inline double State::get_para_max(int i)const
{
  return state_vector[i].get_max();
}
inline double State::get_para_step_size(int i)const
{
  return state_vector[i].get_step_size();
}

inline bool State::get_calc_strength() const
{
  return calc_strength;
}
inline double State::get_strength() const
{
  return strength;
}
inline double State::get_temperature() const
{
  return temperature;
}

inline void State::set_para_label(int i,std::string str)
{
  state_vector[i].set_label(str);
}
inline void State::set_para_min(int i,double n)
{
  state_vector[i].set_min(n);
}
inline void State::set_para_val(int i,double n)
{
  state_vector[i].set_val(n);
}
inline void State::set_para_prop_val(int i,double n)
{
  state_vector[i].set_prop_val(n);
}
inline void State::set_para_max(int i,double n)
{
  state_vector[i].set_max(n);
}
inline void State::set_para_step_size(int i,double n)
{
  state_vector[i].set_step_size(n);
}

inline void State::set_calc_strength(bool b)
{
  calc_strength=b;
}
inline void State::set_strength(double n)
{
  strength=n;
}
inline void State::set_temperature(double n)
{
  temperature=n;
}

inline void State::add_bond(Parameter para)
{
  state_vector.push_back(para);
  prior_vector.push_back(&null);// if there is no prior push a null density
}

inline void State::add_bond(Parameter para,Density* dens)
{
  state_vector.push_back(para);
  prior_vector.push_back(dens);
}
inline void State::add_likelihood(Likelihood* lh)
{
  lik=lh;
}
inline int State::get_no_state()
{
  return state_vector.size();
}
inline double State::get_para_range(int i) const
{
  return state_vector[i].get_range();
}

#endif //UPDATE_H
